/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: libtlen2-p2p.c,v 1.2 2004/02/28 21:39:17 who_ami Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */



#include "libtlen2-p2p.h"

#include <string.h>
#include <sys/time.h>
#ifdef __WIN32__
#include <sys/types.h>
#endif

static GSList *list_p2p;

static
TlenP2p *_tlen_p2p_new (const gchar *id, const gchar *path);



/**
  * tlen_p2p_sendfile:
  * @tc: connection to use.
  * @id: receipent's id.
  * @path: path to a file.
  *
  * Send file to a user using P2P connection.
  *
  * returns: TRUE on success, FALSE otherwise.
  **/
gboolean tlen_p2p_sendfile (TlenConnection *tc, const gchar *id,
                            const gchar *path)
{
  TlenP2p *p;
  gchar *q, *s;
  gsize bytes;
  gboolean ret;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  if (!path)
  {
    tlen_debug ("path == NULL\n");
    return FALSE;
  }
  s = strchr (id, '@');
  if (s)
    *s = '\0';
  p = _tlen_p2p_new (id, path);
  if (!p)
  {
    if (s)
      *s = '@';
    return FALSE;
  }
  _tlen_p2p_add (p);
  q = g_strdup_printf ("<f t=\"%s\" n=\"%s\" e=\"1\" i=\"%s\" c=\"1\" s=\"%d\" v=\"1\"/>",
                       id, p->filename, p->i, p->length);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &bytes);
  if (!ret)
    _tlen_p2p_del (p);
  g_free (q);
  if (s)
    *s = '@';
  return ret;
}

void _tlen_p2p_add (TlenP2p *p)
{
  list_p2p = g_slist_append (list_p2p, p);
}

void _tlen_p2p_del (TlenP2p *p)
{
  list_p2p = g_slist_remove (list_p2p, p);
  _tlen_p2p_free (p);
}

//FIXME ! read in other thread
static
TlenP2p *_tlen_p2p_new (const gchar *id, const gchar *path)
{
  TlenP2p *p;
  gchar *buf = NULL;
  GError *error = NULL;
  gsize bytes;
  gchar *n;
  struct timeval tv;

  if (!g_file_get_contents (path, &buf, &bytes, &error))
  {
    tlen_debug ("Error reading from file '%s'\n", path);
    return NULL;
  }
  tlen_debug ("Read %d bytes of data\n", bytes);
  p = g_new0 (TlenP2p, 1);
  if (p)
  {
    gettimeofday (&tv, NULL);
//    p->i = g_strdup_printf ("%d%d", (gint) tv.tv_usec, (gint) tv.tv_sec);
    p->i = g_strdup_printf ("%d", (gint) tv.tv_sec);
    p->id = g_strdup (id);
    p->buf = buf;
    p->length = bytes;
    n = strrchr (path, G_DIR_SEPARATOR);
    if (n)
      n++;
    else
      n = (gchar*) path;
    p->filename = g_strdup (n);
  }
  return p;
}

void _tlen_p2p_free (TlenP2p *p)
{
  if (p)
  {
    g_free (p->id);
    g_free (p->buf);
    g_free (p->filename);
    g_free (p->i);
    g_free (p);
  }
}

TlenP2p *_tlen_p2p_get (const gchar *id, const gchar *filename)
{
  GSList *l;
  TlenP2p *p;

  l = list_p2p;
  while (l)
  {
    p = (TlenP2p*) l->data;
    if (!filename)
    {
      if (!tlen_strcmp (id, p->id))
        return p;
    }
    else
    {
      if (!tlen_strcmp (id, p->id) && !tlen_strcmp (filename, p->filename))
        return p;
    }
    l = l->next;
  }
  return NULL;
}

